Scriptname JP_NM_MothHandler extends ObjectReference
{Used in lantern activators to dynamically spawn moths}  

; - Properties -
Activator Property MothsEffect Auto
GlobalVariable Property GameMonth Auto
GlobalVariable Property GameHour Auto
GlobalVariable Property SettingSpawnChance Auto
Float Property XOffset = 0.0 Auto
Float Property YOffset = 0.0 Auto
Float Property ZOffset = 0.0 Auto
FormList Property InvalidWorldSpaces Auto

; - Variables -
ObjectReference mothsRef = None
Bool isActive = False
Bool inDeathAnim = False

; - Functions -
Bool Function IsValidWorldSpace()
	if Self.IsInInterior()
		Return false
	else
		if InvalidWorldSpaces.HasForm(Self.GetWorldSpace())
			Return false
		else
			Return true
		endIf
	endIf
EndFunction

Bool Function IsMothTime()
	Int gm = GameMonth.GetValue() As Int
	if gm >= 5 && gm <= 10													; Check game month (May-October)
		Float gh = GameHour.GetValue() As Float
		if (gh >= 22.0 && gh <= 24.0) || (gh >= 0.0 && gh <= 4.0)						; Check time of day (22:00-04:00 o'clock)
			Int weatherClass = Weather.GetCurrentWeather().GetClassification()
			if (weatherClass != 2 && weatherClass != 3)							; Check weather (not Rain or Snow)
				if IsValidWorldSpace()										; Check if valid worldspace
					Return True
				endIf
			endIf
		endIf
	endIf
	Return False
EndFunction

Function PlaceMoths()
	mothsRef = PlaceAtMe(MothsEffect)
	if (XOffset != 0.0 || YOffset != 0.0 || ZOffset != 0.0)
		mothsRef.MoveTo(Self, afXOffset = XOffset, afYOffset = YOffset, afZOffset = ZOffset, abMatchRotation = false)
	endIf
EndFunction

Function TurnOnMoths()
	if !mothsRef
		Int spawnChance = SettingSpawnChance.GetValue() as Int
		if spawnChance >= 100
			PlaceMoths()
		elseIf spawnChance > 0
			int roll = Utility.RandomInt(1, 100)
			if roll <= spawnChance
				PlaceMoths()
			endIf
		endIf
	endIf
EndFunction

Function TurnOffMoths()
	if mothsRef
		mothsRef.Disable()
		mothsRef = mothsRef.Delete()
	endIf
EndFunction

; - Events -
Event OnLoad()
	if IsMothTime()
		TurnOnMoths()
	else
		TurnOffMoths()
	endIf
	isActive = True
	RegisterForSingleUpdateGameTime(1.0)
EndEvent

Event OnUnload()
	TurnOffMoths()
	isActive = False
EndEvent

Event OnUpdateGameTime()
	if isActive
		if IsMothTime()
			TurnOnMoths()
		else
			TurnOffMoths()
		endIf
		RegisterForSingleUpdateGameTime(1.0)
	endIf
EndEvent

Event OnHit(ObjectReference akAggressor, Form akSource, Projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
	if (mothsRef && !inDeathAnim)
		if !inDeathAnim
			inDeathAnim = true
			mothsRef.PlayGamebryoAnimation("FlyOff")
			Utility.Wait(0.9)
			TurnOffMoths()
			inDeathAnim = false
		endIf
	endIf
EndEvent
